"""# ---------------------------------------------------------------------------
 elevation_model.py
 Created on: Tue Nov 17 2009 02:11:08 
 Created by: Jonathan Griffin
 This script compares calculated inundate areas for run-up solutions from analystical models
 and ANUGA modelling.


# ---------------------------------------------------------------------------
"""
# Import system modules
import sys, string, os, arcgisscripting
from os.path import join
from collections import defaultdict
import csv



# Create the Geoprocessor object
gp = arcgisscripting.create(9.3)

# Check out any necessary licenses
gp.CheckOutExtension("spatial")

# Load required toolboxes...
gp.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Spatial Analyst Tools.tbx")
gp.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Conversion Tools.tbx")
gp.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Data Management Tools.tbx")
gp.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Analysis Tools.tbx")

gp.overwriteoutput = 1

filePath = "N:\\georisk_models\\inundation\\data\\new_south_wales\\batemans_bay_tsunami_scenario_2009\\anuga\\outputs\\"
boundary_path = "N:\\georisk_models\\inundation\\data\\new_south_wales\\batemans_bay_tsunami_scenario_2009\\anuga\\boundaries\\"
boundary_file_name = 'wave_energy_summary.csv'
out_file_name = 'area_comparisons.csv'
#figure = 'area_comparisons.png'

boundary_file = join(boundary_path,boundary_file_name)
out_file = join(boundary_path, out_file_name)
#figure_path = join(boundary_path, 'figures',figure)
plot = True

event_dict = {'Event1_MSL':58346,
##              'Event1_HAT':58346,
              'Event2_MSL':51204,
##              'Event2_HAT':51204,
              'Event3_MSL':58284,#,
##              'Event3_HAT':58284,
              'Puysegur_200yr':58129,
              'Puysegur_500yr':58115,
              'Puysegur_1000yr':58226,
              'Puysegur_5000yr':58286,
              'Puysegur_15000yr':58272,
              'New_Hebrides_200yr':51077,
              'New_Hebrides_500yr':51378,
              'New_Hebrides_1000yr':51347,
              'New_Hebrides_2000yr':51292,
              'New_Hebrides_5000yr':51424,}

# Dictionaries for storing area comparison
area_comp = defaultdict(list)
area_percent = defaultdict(list)

boundaries = file(boundary_file,'r')
for line in boundaries.readlines():
    line_split = line.split(',')
    if line_split[0]=='filename':
        continue
    gauge_file = line_split[0]
    event_id = gauge_file.split('/')[10]
    crest_integrated_energy = line_split[1]
    instantaneous_energy = line_split[2]
    run_up_madsen = line_split[3][0:5]
    event_name = False
    for key, value in event_dict.iteritems():
        if value == int(event_id):
            event_name = key
    if event_name == False:
        continue
    raster_path = join(filePath,event_name,'raster.gdb')

    
    print event_id
    print gauge_file
    #break



    
    ##run_up_madsen = 6.0
    run_up_height = str(run_up_madsen)
    run_up_name = run_up_height.replace('.','_')[0]


    ####################################################################################
    ##    Extract areas for analytical solution
    ####################################################################################
    # Local variables...
    elevation = filePath+"elevation\\elevation.gdb\\elevation"
    elevation_extract = filePath+"elevation\\elevation.gdb\\elevation_extract"
    inundation_area1 = "N:\\georisk_models\\inundation\data\\new_south_wales\\batemans_bay_tsunami_scenario_2009\\anuga\\polygons\\polygons.gdb\inundation_area1"
    elevation_reclass = filePath+"elevation\\elevation.gdb\\elevation_"+run_up_name+"_reclass"
    elevation_poly = filePath+"elevation\\features.gdb\\elevation_"+run_up_name+"_poly"
    elevation_less = filePath+"elevation\\features.gdb\\elevation_"+run_up_name+"_less"
    batemans_bay_SMARTLINE = "N:\\georisk_models\\inundation\\data\\new_south_wales\\coastline\\batemans_bay_SMARTLINE.shp"
    elevation_less_final = filePath+"elevation\\features.gdb\\elevation_less_final"

    # Process: Extract elevation data by Mask...
    print 'check if elevation already extracted'
    if gp.Exists(elevation_extract)!=True:
        print 'not already extracted'
        print 'extracting by mask'
        gp.ExtractByMask_sa(elevation, inundation_area1, elevation_extract)
    else:
        print 'extracted elevation already exists'

    # Process: Reclassify elevation data
    print 'check if elevation already reclassified'
    if gp.Exists(elevation_reclass)!=True:
        print 'reclassify based on elevation height of:', run_up_height
        reclass_level = "-200 "+run_up_height+" 1;"+run_up_height+" 300 0"
        gp.Reclassify_sa(elevation_extract, "Value", reclass_level, elevation_reclass, "DATA")
    else:
        print 'elevation has previously been reclassified for this height:', run_up_height

    # Process: Raster to Polygon...
    print 'check if already converted from raster to polyon'
    if gp.Exists(elevation_poly)!=True:
        print 'raster to polyon'
        gp.RasterToPolygon_conversion(elevation_reclass, elevation_poly, "NO_SIMPLIFY", "VALUE")
    else:
        print 'elevation raster already converted to polygon'

    # Process: Select...
    print 'select by attribute'
    gp.Select_analysis(elevation_poly, elevation_less, "\"GRIDCODE\" = 1")

    # Process: Create layer...
    print 'creating layers'
    gp.MakeFeatureLayer(batemans_bay_SMARTLINE,'smartline_layer')
    gp.MakeFeatureLayer(elevation_less,'elevation_layer')

    # Process: Select Layer By Location...
    print 'select by location'
    gp.SelectLayerByLocation_management('elevation_layer', "INTERSECT", 'smartline_layer', "", "NEW_SELECTION")
    print 'joining'
    gp.SpatialJoin_analysis('elevation_layer',inundation_area1, elevation_less_final,"JOIN_ONE_TO_ONE","","","INTERSECTS")

    # Process: Copy Features...
    ##print 'copy features to output feature class'
    ##gp.CopyFeatures_management('elevation_layer', elevation_less_final, "", "0", "0", "0")


    ####################################################################################
    ##    Extract areas for ANUGA solution
    ####################################################################################
    # Local variables...
    gp.workspace = raster_path
    print raster_path
    inundation_name = gp.ListRasters("*depth*","")
    for raster in inundation_name:
        print raster
    inundation = raster_path + "\\"+ inundation_name[0]
    inundation_area1 = "N:\\georisk_models\\inundation\data\\new_south_wales\\batemans_bay_tsunami_scenario_2009\\anuga\\polygons\\polygons.gdb\inundation_area1"
    inundation_extract = "inundation_"+run_up_name+"_extract"
    inundation_reclass = "inundation_"+run_up_name+"_reclass"
    inundation_poly = "inundation_"+run_up_name+"_poly"
    inundation_less = "inundation_"+run_up_name+"_less"
    inundation_less_final = "inundation_less_final"

    # Process: Extract inundation data by Mask...
    print 'check if inundation already extracted'
    if gp.Exists(inundation_extract)!=True:
        print 'not already extracted'
        print 'extracting by mask'
        gp.ExtractByMask_sa(inundation, inundation_area1, inundation_extract)
    else:
        print 'extracted inundation already exists'

    # Process: Reclassify inundation data
    print 'check if inundation already reclassified'
    if gp.Exists(inundation_reclass)!=True:
        print 'reclassify based on inundation'
        gp.Reclassify_sa(inundation_extract, "Value", "-200 0 1;0 300 0", inundation_reclass, "DATA")
    else:
        print 'inundation has previously been reclassified for this height:', run_up_height

    # Process: Raster to Polygon...
    print 'check if already converted from raster to polyon'
    if gp.Exists(inundation_poly)!=True:
        print 'raster to polyon'
        gp.RasterToPolygon_conversion(inundation_reclass, inundation_poly, "NO_SIMPLIFY", "VALUE")
    else:
        print 'inundation raster already converted to polygon'

    # Process: Select...
    print 'select by attribute'
    gp.Select_analysis(inundation_poly, inundation_less, "\"GRIDCODE\" = 0")

    # Process: Create layer...
    print 'creating layers'
    gp.MakeFeatureLayer(batemans_bay_SMARTLINE,'smartline_layer')
    gp.MakeFeatureLayer(inundation_less,'inundation_layer')

    # Process: Select Layer By Location...
    print 'select by location'
    gp.SelectLayerByLocation_management('inundation_layer', "INTERSECT", 'smartline_layer', "", "NEW_SELECTION")
    print 'joining'
    gp.SpatialJoin_analysis('inundation_layer',inundation_area1, inundation_less_final,"JOIN_ONE_TO_ONE","","","INTERSECTS")





    # Search feature class for areas
    print 'get areas'
    analytical_area_dict = {}
    #analytical_areas = []
    cur = gp.SearchCursor(elevation_less_final)
    sRow = cur.Next()
    while sRow:
        if not(sRow.GetValue("inundation_ID")in analytical_area_dict):
            analytical_area_dict[sRow.GetValue("inundation_ID")] = sRow.GetValue("Shape_Area")
        else:
            analytical_area_dict[sRow.GetValue("inundation_ID")] = (analytical_area_dict[sRow.GetValue("inundation_ID")]+sRow.GetValue("Shape_Area"))
        #analytical_areas.append(sRow.GetValue("Shape_Area"))
        sRow = cur.Next()
    #print analytical_areas
    print analytical_area_dict

    # Search feature class for areas
    print 'get areas'
    anuga_area_dict = {}
    #anuga_areas = []
    cur = gp.SearchCursor(inundation_less_final)
    sRow = cur.Next()
    while sRow:
        if not(sRow.GetValue("inundation_ID") in anuga_area_dict):
            anuga_area_dict[sRow.GetValue("inundation_ID")]=sRow.GetValue("Shape_Area")
        else:
            anuga_area_dict[sRow.GetValue("inundation_ID")]= anuga_area_dict[sRow.GetValue("inundation_ID")]+sRow.GetValue("Shape_Area")  
        #anuga_areas.append(sRow.GetValue("Shape_Area"))
        sRow = cur.Next()
    #print anuga_areas
    print anuga_area_dict

    for key in anuga_area_dict:
        if not key in analytical_area_dict:
            analytical_area_dict[key] = 0.
        diff = anuga_area_dict[key]-analytical_area_dict[key]
        percent_diff = 100*diff/anuga_area_dict[key]
        area_comp[key].append(diff)
        area_percent[key].append(percent_diff)
print 'area_comp', area_comp
print 'area_percent', area_percent

##csv_dict = csv.DictWriter(open(out_file,'w'),[1,2,3,4,5,6,7,8,9])
##csv_dict.writerow(dict(zip([1,2,3,4,5,6,7,8,9],[1,2,3,4,5,6,7,8,9])))
##csv_dict.writerow(area_percent)

csv_out = csv.writer(open(out_file,'w'))
csv_out.writerow([1,2,3,4,5,6,7,8,9])
for i in range(len(area_percent[1])):
    value_list =[]
    for key in area_percent:
        value_list.append(area_percent[key][i])
    csv_out.writerow(value_list)
     
out_file.close()    
##for key, value in area_percent:
##    key_list = []
##    for val in value:
##        key_list.append(key)
##    pylab.scatter(key_list,value)
        




boundaries.close()


